# -*- coding: utf-8 -*-
# @time: 2024/3/5 14:35
# @author: Dyz
# @file: jcr_api.py
# @software: PyCharm
# JCR 提供标准API 需要订阅
# app Subscribe https://developer.clarivate.com/applications
# api docs see https://api.clarivate.com/swagger-ui/?url=https://developer.clarivate.com/apis/wos-journal/swagger
